@@ -33,6 +33,7 @@ static guint signals[N_SIGNALS] = { 0, };
 
 struct _TrackerExtractWatchdog {
 	GObject parent_class;
+	gchar *domain;
 	guint extractor_watchdog_id;
 	gboolean initializing;
 };
@@ -95,17 +96,16 @@ static void
 extract_watchdog_start (TrackerExtractWatchdog *watchdog,
 			gboolean                autostart)
 {
-	gchar *domain_name, *tracker_extract_dbus_name;
+	const gchar *domain_name = watchdog->domain;
+	gchar *tracker_extract_dbus_name;
 
 	g_debug ("Setting up watch on tracker-extract (autostart: %s)",
 		 autostart ? "yes" : "no");
 
-	domain_name = tracker_sparql_connection_get_domain ();
-
 	if (domain_name == NULL) {
 		tracker_extract_dbus_name = g_strdup (TRACKER_MINER_DBUS_NAME_PREFIX "Extract");
 	} else {
-		tracker_extract_dbus_name = g_strconcat (domain_name, ".Tracker1.Miner.Extract", NULL);
+		tracker_extract_dbus_name = g_strconcat (domain_name, ".Tracker3.Miner.Extract", NULL);
 	}
 
 	watchdog->extractor_watchdog_id =
@@ -119,7 +119,6 @@ extract_watchdog_start (TrackerExtractWatchdog *watchdog,
 				  watchdog, NULL);
 
 	g_free (tracker_extract_dbus_name);
-	g_free (domain_name);
 }
 
 static void
@@ -128,6 +127,7 @@ tracker_extract_watchdog_finalize (GObject *object)
 	TrackerExtractWatchdog *watchdog = TRACKER_EXTRACT_WATCHDOG (object);
 
 	extract_watchdog_stop (watchdog);
+	g_free (watchdog->domain);
 
 	G_OBJECT_CLASS (tracker_extract_watchdog_parent_class)->finalize (object);
 }
@@ -149,15 +149,21 @@ tracker_extract_watchdog_class_init (TrackerExtractWatchdogClass *klass)
 static void
 tracker_extract_watchdog_init (TrackerExtractWatchdog *watchdog)
 {
-	watchdog->initializing = TRUE;
-	extract_watchdog_start (watchdog, FALSE);
 }
 
 TrackerExtractWatchdog *
-tracker_extract_watchdog_new (void)
+tracker_extract_watchdog_new (const gchar *domain)
 {
-	return g_object_new (TRACKER_TYPE_EXTRACT_WATCHDOG,
-			     NULL);
+	TrackerExtractWatchdog *watchdog;
+
+	watchdog = g_object_new (TRACKER_TYPE_EXTRACT_WATCHDOG,
+	                         NULL);
+
+	watchdog->initializing = TRUE;
+	watchdog->domain = g_strdup (domain);
+	extract_watchdog_start (watchdog, FALSE);
+
+	return watchdog;
 }
 
 void
