@@ -38,7 +38,6 @@
 #include "tracker-config.h"
 #include "tracker-miner-files.h"
 #include "tracker-miner-files-index.h"
-#include "tracker-writeback.h"
 
 #define ABOUT	  \
 	"Tracker " PACKAGE_VERSION "\n"
@@ -99,8 +98,6 @@ log_option_values (TrackerConfig *config)
 		g_message ("General options:");
 		g_message ("  Initial Sleep  ........................  %d",
 		           tracker_config_get_initial_sleep (config));
-		g_message ("  Writeback  ............................  %s",
-		           tracker_config_get_enable_writeback (config) ? "yes" : "no");
 
 		g_message ("Indexer options:");
 		g_message ("  Throttle level  .......................  %d",
@@ -127,14 +124,15 @@ miner_reset_applications (TrackerMiner *miner)
 
 	sparql =
 		"DELETE { ?icon a rdfs:Resource } "
-		"WHERE { ?app a nfo:SoftwareApplication; nfo:softwareIcon ?icon } "
-		"DELETE { ?app nie:dataSource <" TRACKER_PREFIX_TRACKER "extractor-data-source> } "
-		"WHERE { ?app a nfo:SoftwareApplication } ";
+		"WHERE { ?app a nfo:SoftwareApplication; nfo:softwareIcon ?icon }; "
+		"DELETE { ?app tracker:extractorHash ?h } "
+		"WHERE { ?app a nfo:SoftwareApplication ; "
+		"             tracker:extractorHash ?h } ";
 
 	/* Execute a sync update, we don't want the apps miner to start before
 	 * we finish this. */
 	tracker_sparql_connection_update (tracker_miner_get_connection (miner),
-	                                  sparql, G_PRIORITY_HIGH,
+	                                  sparql,
 	                                  NULL, &error);
 
 	if (error) {
@@ -179,6 +177,7 @@ save_current_locale (TrackerDomainOntology *domain_ontology)
 		g_clear_error (&error);
 	}
 
+	g_object_unref (cache);
 	g_free (locale);
 	g_free (locale_file);
 }
@@ -197,6 +196,7 @@ detect_locale_changed (TrackerMiner          *miner,
 	cache = get_cache_dir (domain_ontology);
 	cache_path = g_file_get_path (cache);
 	locale_file = g_build_filename (cache_path, LOCALE_FILENAME, NULL);
+	g_object_unref (cache);
 	g_free (cache_path);
 
 	if (G_LIKELY (g_file_test (locale_file, G_FILE_TEST_EXISTS))) {
@@ -916,23 +916,10 @@ main (gint argc, gchar *argv[])
 		return EXIT_FAILURE;
 	}
 
-	tracker_writeback_init (TRACKER_MINER_FILES (miner_files),
-	                        config,
-	                        &error);
-
-	if (error) {
-		g_critical ("Couldn't create writeback handling: '%s'",
-		            error ? error->message : "unknown error");
-		g_object_unref (config);
-		g_object_unref (miner_files);
-		return EXIT_FAILURE;
-	}
-
 	/* Create new TrackerMinerFilesIndex object */
 	miner_files_index = tracker_miner_files_index_new (TRACKER_MINER_FILES (miner_files));
 	if (!miner_files_index) {
 		g_object_unref (miner_files);
-		tracker_writeback_shutdown ();
 		g_object_unref (config);
 		return EXIT_FAILURE;
 	}
@@ -1012,8 +999,6 @@ main (gint argc, gchar *argv[])
 	g_object_unref (connection);
 	tracker_domain_ontology_unref (domain_ontology);
 
-	tracker_writeback_shutdown ();
-
 	g_print ("\nOK\n\n");
 
 	return EXIT_SUCCESS;
