@@ -29,7 +29,7 @@
 
 static const gchar introspection_xml[] =
   "<node>"
-  "  <interface name='org.freedesktop.Tracker1.Miner.Files.Index'>"
+  "  <interface name='org.freedesktop.Tracker3.Miner.Files.Index'>"
   "    <method name='ReindexMimeTypes'>"
   "      <arg type='as' name='mime_types' direction='in' />"
   "    </method>"
@@ -85,6 +85,37 @@ static void     index_finalize            (GObject              *object);
 
 G_DEFINE_TYPE_WITH_PRIVATE(TrackerMinerFilesIndex, tracker_miner_files_index, G_TYPE_OBJECT)
 
+#define TRACKER_MINER_INDEX_ERROR tracker_miner_index_error_quark ()
+
+GQuark tracker_miner_index_error_quark (void);
+
+typedef enum {
+	TRACKER_MINER_INDEX_ERROR_FILE_NOT_FOUND,
+	TRACKER_MINER_INDEX_ERROR_DIRECTORIES_ONLY,
+	TRACKER_MINER_INDEX_ERROR_NOT_ELIGIBLE,
+	TRACKER_MINER_INDEX_N_ERRORS
+} TrackerMinerIndexError;
+
+static const GDBusErrorEntry tracker_miner_index_error_entries[] =
+{
+	{TRACKER_MINER_INDEX_ERROR_FILE_NOT_FOUND, "org.freedesktop.Tracker.Miner.Files.Index.Error.FileNotFound"},
+	{TRACKER_MINER_INDEX_ERROR_DIRECTORIES_ONLY, "org.freedesktop.Tracker.Miner.Files.Index.Error.DirectoriesOnly"},
+	{TRACKER_MINER_INDEX_ERROR_NOT_ELIGIBLE, "org.freedesktop.Tracker.Miner.Files.Index.Error.NotEligible"},
+};
+
+G_STATIC_ASSERT (G_N_ELEMENTS (tracker_miner_index_error_entries) == TRACKER_MINER_INDEX_N_ERRORS);
+
+GQuark
+tracker_miner_index_error_quark (void)
+{
+	static volatile gsize quark_volatile = 0;
+	g_dbus_error_register_error_domain ("tracker-miner-index-error-quark",
+	                                    &quark_volatile,
+	                                    tracker_miner_index_error_entries,
+	                                    G_N_ELEMENTS (tracker_miner_index_error_entries));
+	return (GQuark) quark_volatile;
+}
+
 static void
 tracker_miner_files_index_class_init (TrackerMinerFilesIndexClass *klass)
 {
@@ -98,11 +129,11 @@ tracker_miner_files_index_class_init (TrackerMinerFilesIndexClass *klass)
 
 	g_object_class_install_property (object_class,
 	                                 PROP_FILES_MINER,
-	                                 g_param_spec_object ("files_miner",
-	                                                      "files_miner",
+	                                 g_param_spec_object ("files-miner",
+	                                                      "files-miner",
 	                                                      "The FS Miner",
 	                                                      TRACKER_TYPE_MINER_FILES,
-	                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -247,7 +278,6 @@ tracker_miner_files_index_reindex_mime_types (TrackerMinerFilesIndex *miner,
 {
 	TrackerMinerFilesIndexPrivate *priv;
 	GString *query;
-	GError *inner_error = NULL;
 	TrackerSparqlConnection *connection;
 	TrackerDBusRequest *request;
 	gint len, i;
@@ -265,15 +295,7 @@ tracker_miner_files_index_reindex_mime_types (TrackerMinerFilesIndex *miner,
 	                                        __FUNCTION__,
 	                                        len);
 
-	connection = tracker_sparql_connection_get (NULL, &inner_error);
-
-	if (!connection) {
-		g_free (mime_types);
-		tracker_dbus_request_end (request, inner_error);
-		g_dbus_method_invocation_return_gerror (invocation, inner_error);
-		g_error_free (inner_error);
-		return;
-	}
+	connection = tracker_miner_get_connection (TRACKER_MINER (priv->files_miner));
 
 	tracker_dbus_request_comment (request,
 	                              "Attempting to reindex the following mime types:");
@@ -281,7 +303,7 @@ tracker_miner_files_index_reindex_mime_types (TrackerMinerFilesIndex *miner,
 	query = g_string_new ("SELECT ?url "
 	                      "WHERE {"
 	                      "  ?resource nie:url ?url ;"
-	                      "  nie:mimeType ?mime ."
+	                      "  nie:interpretedAs/nie:mimeType ?mime ."
 	                      "  FILTER(");
 
 	for (i = 0; i < len; i++) {
@@ -341,7 +363,9 @@ handle_method_call_index_file (TrackerMinerFilesIndex *miner,
 	                               NULL, NULL);
 
 	if (!file_info) {
-		internal_error = g_error_new_literal (1, 0, "File does not exist");
+		internal_error = g_error_new_literal (TRACKER_MINER_INDEX_ERROR,
+		                                      TRACKER_MINER_INDEX_ERROR_FILE_NOT_FOUND,
+		                                      "File does not exist");
 		tracker_dbus_request_end (request, internal_error);
 		g_dbus_method_invocation_return_gerror (invocation, internal_error);
 
@@ -358,7 +382,9 @@ handle_method_call_index_file (TrackerMinerFilesIndex *miner,
 #ifdef REQUIRE_LOCATION_IN_CONFIG
 	do_checks = TRUE;
 	if (!tracker_miner_files_is_file_eligible (priv->files_miner, file)) {
-		internal_error = g_error_new_literal (1, 0, "File is not eligible to be indexed");
+		internal_error = g_error_new_literal (TRACKER_MINER_INDEX_ERROR,
+		                                      TRACKER_MINER_INDEX_ERROR_NOT_ELIGIBLE,
+		                                      "File is not eligible to be indexed");
 		tracker_dbus_request_end (request, internal_error);
 		g_dbus_method_invocation_return_gerror (invocation, internal_error);
 
@@ -546,9 +572,9 @@ tracker_miner_files_index_new (TrackerMinerFiles *miner_files)
 	/* Register the service name for the miner */
 	full_path = g_strconcat (TRACKER_MINER_DBUS_PATH_PREFIX, "Files/Index", NULL);
 
-	g_message ("Registering D-Bus object...");
-	g_message ("  Path:'%s'", full_path);
-	g_message ("  Object Type:'%s'", G_OBJECT_TYPE_NAME (miner));
+	g_debug ("Registering D-Bus object...");
+	g_debug ("  Path:'%s'", full_path);
+	g_debug ("  Object Type:'%s'", G_OBJECT_TYPE_NAME (miner));
 
 	priv->registration_id =
 		g_dbus_connection_register_object (priv->d_connection,
